# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

from logging import getLogger
from threading import Thread
from time import time, sleep

from requests import post

from tb_rest_client32.api.api_ce import *
from tb_rest_client32.api_client import ApiClient
from tb_rest_client32.configuration import Configuration
from tb_rest_client32.models.models_ce import *

logger = getLogger(__name__)


class RestClientBase(Thread):
    def __init__(self, base_url):
        super().__init__()
        if base_url.startswith("http"):
            self.base_url = base_url
        else:
            self.base_url = "http://" + base_url
        self.token_info = {"token": "", "refreshToken": 0}
        self.api_client = None
        self.username = None
        self.password = None
        self.logged_in = False
        self.stopped = True
        self.configuration = Configuration()
        self.configuration.host = self.base_url

    def run(self):
        self.stopped = False
        while not self.stopped:
            try:
                check_time = time()
                if check_time >= self.token_info["refreshToken"] and self.logged_in:
                    if self.username and self.password:
                        self.login(self.username, self.password)
                    else:
                        logger.error("No username or password provided!")
                sleep(1)
            except Exception as e:
                logger.exception(e)
                break
            except KeyboardInterrupt:
                break

    def stop(self):
        self.stopped = True

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()

    def login(self, username, password):
        """Authorization on the host and saving the toke information"""
        if self.username is None and self.password is None:
            self.username = username
            self.password = password
            self.logged_in = True

        token_json = post(self.base_url + "/api/auth/login",
                          json={"username": username, "password": password},
                          verify=self.configuration.verify_ssl).json()
        token = None
        if isinstance(token_json, dict) and token_json.get("token") is not None:
            token = token_json["token"]
        self.configuration.api_key_prefix["X-Authorization"] = "Bearer"
        self.configuration.api_key["X-Authorization"] = token

        self.api_client = ApiClient(self.configuration)
        self.__load_controllers()

    def get_token(self):
        return self.token_info["token"]

    def delete_asset(self, asset_id: AssetId, **kwargs):
        return self.asset_controller.delete_asset_using_delete(asset_id.id, **kwargs)

    def get_asset_by_id(self, asset_id: AssetId, **kwargs):
        return self.asset_controller.get_asset_by_id_using_get(asset_id.id, **kwargs)

    def get_asset_types(self, **kwargs):
        return self.asset_controller.get_asset_types_using_get(**kwargs)

    def get_assets_by_ids(self, asset_ids, **kwargs):
        return self.asset_controller.get_assets_by_ids_using_get(asset_ids, **kwargs)

    def get_customer_assets(self, customer_id: CustomerId, page_size=100, page=0, **kwargs):
        return self.asset_controller.get_customer_assets_using_get(customer_id.id, str(page_size), str(page), **kwargs)

    def get_tenant_asset(self, asset_name, **kwargs):
        return self.asset_controller.get_tenant_asset_using_get(asset_name, **kwargs)

    def get_tenant_assets(self, page_size=100, page=0, **kwargs):
        return self.asset_controller.get_tenant_assets_using_get(str(page_size), str(page), **kwargs)

    def save_asset(self, asset, **kwargs):
        return self.asset_controller.save_asset_using_post(asset, **kwargs)

    ##################################

    def delete_user(self, user_id: UserId, **kwargs):
        return self.user_controller.delete_user_using_delete(user_id.id, **kwargs)

    def get_activation_link(self, user_id: UserId, **kwargs):
        return self.user_controller.get_activation_link_using_get(user_id.id, **kwargs)

    def get_customer_users(self, customer_id: CustomerId, page_size=100, page=0, **kwargs):
        return self.user_controller.get_customer_users_using_get(customer_id.id, str(page_size), str(page), **kwargs)

    def get_tenant_admins(self, tenant_id: TenantId, page_size=100, page=0, **kwargs):
        return self.user_controller.get_tenant_admins_using_get(tenant_id.id, str(page_size), str(page), **kwargs)

    def get_user_by_id(self, user_id: UserId, **kwargs):
        return self.user_controller.get_user_by_id_using_get(user_id.id, **kwargs)

    def get_user_token(self, user_id: UserId, **kwargs):
        return self.user_controller.get_user_token_using_get(user_id.id, **kwargs)

    def is_user_token_access_enabled(self, **kwargs):
        return self.user_controller.is_user_token_access_enabled_using_get(**kwargs)

    def save_user(self, user, **kwargs):
        return self.user_controller.save_user_using_post(user, **kwargs)

    def send_activation_email(self, email, **kwargs):
        return self.user_controller.send_activation_email_using_post(email, **kwargs)

    def set_user_credentials_enabled(self, user_id: UserId, **kwargs):
        return self.user_controller.set_user_credentials_enabled_using_post(user_id.id, **kwargs)

    ##################################

    def get_activation_token(self, user_id: UserId):
        user_id = self.get_id(user_id)
        return self.user_controller.get_activation_token_using_get(user_id)

    def activate_user(self, user_id: UserId, password):
        user_id = self.get_id(user_id)
        activation_request = {"activateToken": self.get_activation_token(user_id).replace("'", '"'),
                              "password": password}
        return self.auth_controller.activate_user_using_post(activation_request)

    def change_password(self, change_password_request, **kwargs):
        return self.auth_controller.change_password_using_post(change_password_request, **kwargs)

    def check_activate_token(self, activate_token, **kwargs):
        return self.auth_controller.check_activate_token_using_get(activate_token, **kwargs)

    def check_reset_token(self, reset_token, **kwargs):
        return self.auth_controller.check_reset_token_using_get(reset_token, **kwargs)

    def get_user_password_policy(self, **kwargs):
        return self.auth_controller.get_user_password_policy_using_get(**kwargs)

    def get_user(self, **kwargs):
        return self.auth_controller.get_user_using_get(**kwargs)

    def logout(self, **kwargs):
        return self.auth_controller.logout_using_post(**kwargs)

    def request_reset_password_by_email(self, reset_password_by_email_request, **kwargs):
        return self.auth_controller.request_reset_password_by_email_using_post(reset_password_by_email_request, **kwargs)

    def reset_password(self, reset_password_request, **kwargs):
        return self.auth_controller.reset_password_using_post(reset_password_request, **kwargs)

    ##################################

    def assign_device_to_tenant(self, tenant_id: TenantId, device_id: DeviceId, **kwargs):
        return self.device_controller.assign_device_to_tenant_using_post(tenant_id.id, device_id.id, **kwargs)

    def claim_device1(self, device_name, **kwargs):
        return self.device_controller.claim_device_using_post1(device_name, **kwargs)

    def delete_device(self, device_id: DeviceId, **kwargs):
        return self.device_controller.delete_device_using_delete(device_id.id, **kwargs)

    def get_customer_devices(self, customer_id: CustomerId, page_size=100, page=0, **kwargs):
        return self.device_controller.get_customer_devices_using_get(customer_id.id, str(page_size), str(page), **kwargs)

    def get_device_by_id(self, device_id: DeviceId, **kwargs):
        return self.device_controller.get_device_by_id_using_get(device_id.id, **kwargs)

    def get_device_credentials_by_device_id(self, device_id: DeviceId, **kwargs):
        return self.device_controller.get_device_credentials_by_device_id_using_get(device_id.id, **kwargs)

    def get_device_types(self, **kwargs):
        return self.device_controller.get_device_types_using_get(**kwargs)

    def get_devices_by_ids(self, device_ids, **kwargs):
        return self.device_controller.get_devices_by_ids_using_get(device_ids, **kwargs)

    def get_tenant_device(self, device_name, **kwargs):
        return self.device_controller.get_tenant_device_using_get(device_name, **kwargs)

    def get_tenant_devices(self, page_size=100, page=0, **kwargs):
        return self.device_controller.get_tenant_devices_using_get(str(page_size), str(page), **kwargs)

    def re_claim_device(self, device_name, **kwargs):
        return self.device_controller.re_claim_device_using_delete(device_name, **kwargs)

    def save_device_credentials(self, device_credentials, **kwargs):
        return self.device_controller.save_device_credentials_using_post(device_credentials, **kwargs)

    def save_device(self, device, **kwargs):
        return self.device_controller.save_device_using_post(device, **kwargs)

    ##################################

    def delete_widget_type(self, widget_type_id: WidgetTypeId, **kwargs):
        return self.widget_type_controller.delete_widget_type_using_delete(widget_type_id.id, **kwargs)

    def get_bundle_widget_types(self, is_system, bundle_alias, **kwargs):
        return self.widget_type_controller.get_bundle_widget_types_using_get(is_system, bundle_alias, **kwargs)

    def get_widget_type_by_id(self, widget_type_id: WidgetTypeId, **kwargs):
        return self.widget_type_controller.get_widget_type_by_id_using_get(widget_type_id.id, **kwargs)

    def get_widget_type(self, is_system, bundle_alias, alias, **kwargs):
        return self.widget_type_controller.get_widget_type_using_get(is_system, bundle_alias, alias, **kwargs)

    def save_widget_type(self, widget_type, **kwargs):
        return self.widget_type_controller.save_widget_type_using_post(widget_type, **kwargs)

    ##################################

    def delete_entity_attributes(self, device_id: DeviceId, scope, keys, **kwargs):
        return self.telemetry_controller.delete_entity_attributes_using_delete(device_id.id, scope, keys, **kwargs)

    def delete_entity_attributes1(self, entity_id: EntityId, scope, keys, **kwargs):
        return self.telemetry_controller.delete_entity_attributes_using_delete1(entity_id.entity_type, entity_id.id, scope, keys, **kwargs)

    def delete_entity_timeseries(self, entity_id: EntityId, keys, **kwargs):
        return self.telemetry_controller.delete_entity_timeseries_using_delete(entity_id.entity_type, entity_id.id, keys, **kwargs)

    def get_attribute_keys_by_scope(self, entity_id: EntityId, scope, **kwargs):
        return self.telemetry_controller.get_attribute_keys_by_scope_using_get(entity_id.entity_type, entity_id.id, scope, **kwargs)

    def get_attribute_keys(self, entity_id: EntityId, **kwargs):
        return self.telemetry_controller.get_attribute_keys_using_get(entity_id.entity_type, entity_id.id, **kwargs)

    def get_attributes_by_scope(self, entity_id: EntityId, scope, **kwargs):
        return self.telemetry_controller.get_attributes_by_scope_using_get(entity_id.entity_type, entity_id.id, scope, **kwargs)

    def get_attributes(self, entity_id: EntityId, **kwargs):
        return self.telemetry_controller.get_attributes_using_get(entity_id.entity_type, entity_id.id, **kwargs)

    def get_latest_timeseries(self, entity_id: EntityId, **kwargs):
        return self.telemetry_controller.get_latest_timeseries_using_get(entity_id.entity_type, entity_id.id, **kwargs)

    def get_timeseries_keys(self, entity_id: EntityId, **kwargs):
        return self.telemetry_controller.get_timeseries_keys_using_get(entity_id.entity_type, entity_id.id, **kwargs)

    def get_timeseries(self, entity_id: EntityId, keys, start_ts, end_ts, **kwargs):
        return self.telemetry_controller.get_timeseries_using_get(entity_id.entity_type, entity_id.id, keys, start_ts, end_ts, **kwargs)

    def save_device_attributes(self, device_id: DeviceId, scope, request, **kwargs):
        return self.telemetry_controller.save_device_attributes_using_post(device_id.id, scope, request, **kwargs)

    def save_entity_attributes_v1(self, entity_id: EntityId, scope, request, **kwargs):
        return self.telemetry_controller.save_entity_attributes_v1_using_post(entity_id.entity_type, entity_id.id, scope, request, **kwargs)

    def save_entity_attributes_v2(self, entity_id: EntityId, scope, request, **kwargs):
        return self.telemetry_controller.save_entity_attributes_v2_using_post(entity_id.entity_type, entity_id.id, scope, request, **kwargs)

    def save_entity_telemetry(self, entity_id: EntityId, scope, request_body, **kwargs):
        return self.telemetry_controller.save_entity_telemetry_using_post(entity_id.entity_type, entity_id.id, scope, request_body, **kwargs)

    def save_entity_telemetry_with_ttl(self, entity_id: EntityId, scope, ttl, request_body, **kwargs):
        return self.telemetry_controller.save_entity_telemetry_with_ttl_using_post(entity_id.entity_type, entity_id.id, scope, ttl, request_body, **kwargs)

    ##################################

    def get_tenant_queues_by_service_type(self, service_type, **kwargs):
        return self.queue_controller.get_tenant_queues_by_service_type_using_get(service_type, **kwargs)

    ##################################

    def get_events(self, entity_id: EntityId, event_type, tenant_id: TenantId, page_size=100, page=0, **kwargs):
        return self.event_controller.get_events_using_get(entity_id.entity_type, entity_id.id, event_type, tenant_id.id, str(page_size), str(page), **kwargs)

    def get_events1(self, entity_id: EntityId, tenant_id: TenantId, page_size=100, page=0, **kwargs):
        return self.event_controller.get_events_using_get1(entity_id.entity_type, entity_id.id, tenant_id.id, str(page_size), str(page), **kwargs)

    ##################################

    def delete_rule_chain(self, rule_chain_id: RuleChainId, **kwargs):
        return self.rule_chain_controller.delete_rule_chain_using_delete(rule_chain_id.id, **kwargs)

    def export_rule_chains(self, limit, **kwargs):
        return self.rule_chain_controller.export_rule_chains_using_get(limit, **kwargs)

    def get_latest_rule_node_debug_input(self, rule_node_id: RuleNodeId, **kwargs):
        return self.rule_chain_controller.get_latest_rule_node_debug_input_using_get(rule_node_id.id, **kwargs)

    def get_rule_chain_by_id(self, rule_chain_id: RuleChainId, **kwargs):
        return self.rule_chain_controller.get_rule_chain_by_id_using_get(rule_chain_id.id, **kwargs)

    def get_rule_chain_meta_data(self, rule_chain_id: RuleChainId, **kwargs):
        return self.rule_chain_controller.get_rule_chain_meta_data_using_get(rule_chain_id.id, **kwargs)

    def get_rule_chains(self, page_size=100, page=0, **kwargs):
        return self.rule_chain_controller.get_rule_chains_using_get(str(page_size), str(page), **kwargs)

    def import_rule_chains(self, rule_chain_data, **kwargs):
        return self.rule_chain_controller.import_rule_chains_using_post(rule_chain_data, **kwargs)

    def save_rule_chain_meta_data(self, rule_chain_meta_data, **kwargs):
        return self.rule_chain_controller.save_rule_chain_meta_data_using_post(rule_chain_meta_data, **kwargs)

    def save_rule_chain(self, request, **kwargs):
        return self.rule_chain_controller.save_rule_chain_using_post(request, **kwargs)

    def save_rule_chain1(self, rule_chain, **kwargs):
        return self.rule_chain_controller.save_rule_chain_using_post1(rule_chain, **kwargs)

    def set_root_rule_chain(self, rule_chain_id: RuleChainId, **kwargs):
        return self.rule_chain_controller.set_root_rule_chain_using_post(rule_chain_id.id, **kwargs)

    def test_script(self, input_params, **kwargs):
        return self.rule_chain_controller.test_script_using_post(input_params, **kwargs)

    ##################################

    def count_entities_by_query(self, query, **kwargs):
        return self.entity_query_controller.count_entities_by_query_using_post(query, **kwargs)

    def find_alarm_data_by_query(self, query, **kwargs):
        return self.entity_query_controller.find_alarm_data_by_query_using_post(query, **kwargs)

    def find_entity_data_by_query(self, query, **kwargs):
        return self.entity_query_controller.find_entity_data_by_query_using_post(query, **kwargs)

    ##################################

    def delete_tenant(self, tenant_id: TenantId, **kwargs):
        return self.tenant_controller.delete_tenant_using_delete(tenant_id.id, **kwargs)

    def get_tenant_by_id(self, tenant_id: TenantId, **kwargs):
        return self.tenant_controller.get_tenant_by_id_using_get(tenant_id.id, **kwargs)

    def get_tenant_info_by_id(self, tenant_id: TenantId, **kwargs):
        return self.tenant_controller.get_tenant_info_by_id_using_get(tenant_id.id, **kwargs)

    def get_tenant_infos(self, page_size=100, page=0, **kwargs):
        return self.tenant_controller.get_tenant_infos_using_get(str(page_size), str(page), **kwargs)

    def get_tenants(self, page_size=100, page=0, **kwargs):
        return self.tenant_controller.get_tenants_using_get(str(page_size), str(page), **kwargs)

    def save_tenant(self, tenant, **kwargs):
        return self.tenant_controller.save_tenant_using_post(tenant, **kwargs)

    ##################################

    def delete_device_profile(self, device_profile_id: DeviceProfileId, **kwargs):
        return self.device_profile_controller.delete_device_profile_using_delete(device_profile_id.id, **kwargs)

    def get_default_device_profile_info(self, **kwargs):
        return self.device_profile_controller.get_default_device_profile_info_using_get(**kwargs)

    def get_device_profile_by_id(self, device_profile_id: DeviceProfileId, **kwargs):
        return self.device_profile_controller.get_device_profile_by_id_using_get(device_profile_id.id, **kwargs)

    def get_device_profile_info_by_id(self, device_profile_id: DeviceProfileId, **kwargs):
        return self.device_profile_controller.get_device_profile_info_by_id_using_get(device_profile_id.id, **kwargs)

    def get_device_profile_infos(self, page_size=100, page=0, **kwargs):
        return self.device_profile_controller.get_device_profile_infos_using_get(str(page_size), str(page), **kwargs)

    def get_device_profiles(self, page_size=100, page=0, **kwargs):
        return self.device_profile_controller.get_device_profiles_using_get(str(page_size), str(page), **kwargs)

    def save_device_profile(self, device_profile, **kwargs):
        return self.device_profile_controller.save_device_profile_using_post(device_profile, **kwargs)

    def set_default_device_profile(self, device_profile_id: DeviceProfileId, **kwargs):
        return self.device_profile_controller.set_default_device_profile_using_post(device_profile_id.id, **kwargs)

    ##################################

    def delete_dashboard(self, dashboard_id: DashboardId, **kwargs):
        return self.dashboard_controller.delete_dashboard_using_delete(dashboard_id.id, **kwargs)

    def get_dashboard_by_id(self, dashboard_id: DashboardId, **kwargs):
        return self.dashboard_controller.get_dashboard_by_id_using_get(dashboard_id.id, **kwargs)

    def get_dashboard_info_by_id(self, dashboard_id: DashboardId, **kwargs):
        return self.dashboard_controller.get_dashboard_info_by_id_using_get(dashboard_id.id, **kwargs)

    def get_max_datapoints_limit(self, **kwargs):
        return self.dashboard_controller.get_max_datapoints_limit_using_get(**kwargs)

    def get_server_time(self, **kwargs):
        return self.dashboard_controller.get_server_time_using_get(**kwargs)

    def get_tenant_dashboards(self, page_size=100, page=0, **kwargs):
        return self.dashboard_controller.get_tenant_dashboards_using_get(str(page_size), str(page), **kwargs)

    def get_tenant_dashboards1(self, tenant_id: TenantId, page_size=100, page=0, **kwargs):
        return self.dashboard_controller.get_tenant_dashboards_using_get1(tenant_id.id, str(page_size), str(page), **kwargs)

    def save_dashboard(self, dashboard, **kwargs):
        return self.dashboard_controller.save_dashboard_using_post(dashboard, **kwargs)

    ##################################

    def claim_device(self, device_token, **kwargs):
        return self.device_api_controller.claim_device_using_post(device_token, **kwargs)

    def get_device_attributes(self, device_token, **kwargs):
        return self.device_api_controller.get_device_attributes_using_get(device_token, **kwargs)

    def post_device_attributes(self, device_token, json, **kwargs):
        return self.device_api_controller.post_device_attributes_using_post(device_token, json, **kwargs)

    def post_rpc_request(self, device_token, json, **kwargs):
        return self.device_api_controller.post_rpc_request_using_post(device_token, json, **kwargs)

    def post_telemetry(self, device_token, json, **kwargs):
        return self.device_api_controller.post_telemetry_using_post(device_token, json, **kwargs)

    def provision_device(self, json, **kwargs):
        return self.device_api_controller.provision_device_using_post(json, **kwargs)

    def reply_to_command(self, device_token, request_id, json, **kwargs):
        return self.device_api_controller.reply_to_command_using_post(device_token, request_id, json, **kwargs)

    def subscribe_to_attributes(self, device_token, **kwargs):
        return self.device_api_controller.subscribe_to_attributes_using_get(device_token, **kwargs)

    def subscribe_to_commands(self, device_token, **kwargs):
        return self.device_api_controller.subscribe_to_commands_using_get(device_token, **kwargs)

    ##################################

    def delete_relation(self, from_id, from_type, relation_type, to_id, to_type, **kwargs):
        return self.entity_relation_controller.delete_relation_using_delete(from_id, from_type, relation_type, to_id, to_type, **kwargs)

    def delete_relations(self, entity_id: EntityId, id, type, **kwargs):
        return self.entity_relation_controller.delete_relations_using_delete(entity_id.id, entity_id.entity_type, id, type, **kwargs)

    def find_info_by_from(self, from_id, from_type, **kwargs):
        return self.entity_relation_controller.find_info_by_from_using_get(from_id, from_type, **kwargs)

    def find_info_by_query(self, query, **kwargs):
        return self.entity_relation_controller.find_info_by_query_using_post(query, **kwargs)

    def find_info_by_to(self, to_id, to_type, **kwargs):
        return self.entity_relation_controller.find_info_by_to_using_get(to_id, to_type, **kwargs)

    def get_relation(self, from_id, from_type, relation_type, to_id, to_type, **kwargs):
        return self.entity_relation_controller.get_relation_using_get(from_id, from_type, relation_type, to_id, to_type, **kwargs)

    def save_relation(self, relation, **kwargs):
        return self.entity_relation_controller.save_relation_using_post(relation, **kwargs)

    ##################################

    def handle_one_way_device_rpc_request(self, device_id: DeviceId, request_body, **kwargs):
        return self.rpc_controller.handle_one_way_device_rpc_request_using_post(device_id.id, request_body, **kwargs)

    def handle_two_way_device_rpc_request(self, device_id: DeviceId, request_body, **kwargs):
        return self.rpc_controller.handle_two_way_device_rpc_request_using_post(device_id.id, request_body, **kwargs)

    ##################################

    def ack_alarm(self, alarm_id: AlarmId, **kwargs):
        return self.alarm_controller.ack_alarm_using_post(alarm_id.id, **kwargs)

    def clear_alarm(self, alarm_id: AlarmId, **kwargs):
        return self.alarm_controller.clear_alarm_using_post(alarm_id.id, **kwargs)

    def delete_alarm(self, alarm_id: AlarmId, **kwargs):
        return self.alarm_controller.delete_alarm_using_delete(alarm_id.id, **kwargs)

    def get_alarm_by_id(self, alarm_id: AlarmId, **kwargs):
        return self.alarm_controller.get_alarm_by_id_using_get(alarm_id.id, **kwargs)

    def get_alarm_info_by_id(self, alarm_id: AlarmId, **kwargs):
        return self.alarm_controller.get_alarm_info_by_id_using_get(alarm_id.id, **kwargs)

    def get_alarms(self, entity_id: EntityId, page_size=100, page=0, **kwargs):
        return self.alarm_controller.get_alarms_using_get(entity_id.entity_type, entity_id.id, str(page_size), str(page), **kwargs)

    def get_highest_alarm_severity(self, entity_id: EntityId, **kwargs):
        return self.alarm_controller.get_highest_alarm_severity_using_get(entity_id.entity_type, entity_id.id, **kwargs)

    def save_alarm(self, alarm, **kwargs):
        return self.alarm_controller.save_alarm_using_post(alarm, **kwargs)

    ##################################

    def delete_widgets_bundle(self, widgets_bundle_id: WidgetsBundleId, **kwargs):
        return self.widgets_bundle_controller.delete_widgets_bundle_using_delete(widgets_bundle_id.id, **kwargs)

    def get_widgets_bundle_by_id(self, widgets_bundle_id: WidgetsBundleId, **kwargs):
        return self.widgets_bundle_controller.get_widgets_bundle_by_id_using_get(widgets_bundle_id.id, **kwargs)

    def get_widgets_bundles(self, page_size=100, page=0, **kwargs):
        return self.widgets_bundle_controller.get_widgets_bundles_using_get(str(page_size), str(page), **kwargs)

    def get_widgets_bundles1(self, **kwargs):
        return self.widgets_bundle_controller.get_widgets_bundles_using_get1(**kwargs)

    def save_widgets_bundle(self, widgets_bundle, **kwargs):
        return self.widgets_bundle_controller.save_widgets_bundle_using_post(widgets_bundle, **kwargs)

    ##################################

    def delete_tenant_profile(self, tenant_profile_id: TenantProfileId, **kwargs):
        return self.tenant_profile_controller.delete_tenant_profile_using_delete(tenant_profile_id.id, **kwargs)

    def get_default_tenant_profile_info(self, **kwargs):
        return self.tenant_profile_controller.get_default_tenant_profile_info_using_get(**kwargs)

    def get_tenant_profile_by_id(self, tenant_profile_id: TenantProfileId, **kwargs):
        return self.tenant_profile_controller.get_tenant_profile_by_id_using_get(tenant_profile_id.id, **kwargs)

    def get_tenant_profile_info_by_id(self, tenant_profile_id: TenantProfileId, **kwargs):
        return self.tenant_profile_controller.get_tenant_profile_info_by_id_using_get(tenant_profile_id.id, **kwargs)

    def get_tenant_profile_infos(self, page_size=100, page=0, **kwargs):
        return self.tenant_profile_controller.get_tenant_profile_infos_using_get(str(page_size), str(page), **kwargs)

    def get_tenant_profiles(self, page_size=100, page=0, **kwargs):
        return self.tenant_profile_controller.get_tenant_profiles_using_get(str(page_size), str(page), **kwargs)

    def save_tenant_profile(self, tenant_profile, **kwargs):
        return self.tenant_profile_controller.save_tenant_profile_using_post(tenant_profile, **kwargs)

    def set_default_tenant_profile(self, tenant_profile_id: TenantProfileId, **kwargs):
        return self.tenant_profile_controller.set_default_tenant_profile_using_post(tenant_profile_id.id, **kwargs)

    ##################################

    def get_component_descriptor_by_clazz(self, component_descriptor_clazz, **kwargs):
        return self.component_descriptor_controller.get_component_descriptor_by_clazz_using_get(component_descriptor_clazz, **kwargs)

    def get_component_descriptors_by_type(self, component_type, **kwargs):
        return self.component_descriptor_controller.get_component_descriptors_by_type_using_get(component_type, **kwargs)

    def get_component_descriptors_by_types(self, component_types, **kwargs):
        return self.component_descriptor_controller.get_component_descriptors_by_types_using_get(component_types, **kwargs)

    ##################################

    def get_audit_logs_by_customer_id(self, customer_id: CustomerId, page_size=100, page=0, **kwargs):
        return self.audit_log_controller.get_audit_logs_by_customer_id_using_get(customer_id.id, str(page_size), str(page), **kwargs)

    def get_audit_logs_by_entity_id(self, entity_id: EntityId, page_size=100, page=0, **kwargs):
        return self.audit_log_controller.get_audit_logs_by_entity_id_using_get(entity_id.entity_type, entity_id.id, str(page_size), str(page), **kwargs)

    def get_audit_logs_by_user_id(self, user_id: UserId, page_size=100, page=0, **kwargs):
        return self.audit_log_controller.get_audit_logs_by_user_id_using_get(user_id.id, str(page_size), str(page), **kwargs)

    def get_audit_logs(self, page_size=100, page=0, **kwargs):
        return self.audit_log_controller.get_audit_logs_using_get(str(page_size), str(page), **kwargs)

    ##################################

    def get_current_o_auth2_params(self, **kwargs):
        return self.o_auth_2_controller.get_current_o_auth2_params_using_get(**kwargs)

    def get_login_processing_url(self, **kwargs):
        return self.o_auth_2_controller.get_login_processing_url_using_get(**kwargs)

    def get_o_auth2_clients(self, **kwargs):
        return self.o_auth_2_controller.get_o_auth2_clients_using_post(**kwargs)

    def save_o_auth2_params(self, oauth2_params, **kwargs):
        return self.o_auth_2_controller.save_o_auth2_params_using_post(oauth2_params, **kwargs)

    ##################################

    def delete_client_registration_template(self, client_registration_template_id, **kwargs):
        return self.o_auth_2_config_template_controller.delete_client_registration_template_using_delete(client_registration_template_id, **kwargs)

    def get_client_registration_templates(self, **kwargs):
        return self.o_auth_2_config_template_controller.get_client_registration_templates_using_get(**kwargs)

    def save_client_registration_template(self, client_registration_template, **kwargs):
        return self.o_auth_2_config_template_controller.save_client_registration_template_using_post(client_registration_template, **kwargs)

    ##################################

    def delete_entity_view(self, entity_view_id: EntityViewId, **kwargs):
        return self.entity_view_controller.delete_entity_view_using_delete(entity_view_id.id, **kwargs)

    def get_customer_entity_views(self, customer_id: CustomerId, page_size=100, page=0, **kwargs):
        return self.entity_view_controller.get_customer_entity_views_using_get(customer_id.id, str(page_size), str(page), **kwargs)

    def get_entity_view_by_id(self, entity_view_id: EntityViewId, **kwargs):
        return self.entity_view_controller.get_entity_view_by_id_using_get(entity_view_id.id, **kwargs)

    def get_entity_view_types(self, **kwargs):
        return self.entity_view_controller.get_entity_view_types_using_get(**kwargs)

    def get_tenant_entity_view(self, entity_view_name, **kwargs):
        return self.entity_view_controller.get_tenant_entity_view_using_get(entity_view_name, **kwargs)

    def get_tenant_entity_views(self, page_size=100, page=0, **kwargs):
        return self.entity_view_controller.get_tenant_entity_views_using_get(str(page_size), str(page), **kwargs)

    def save_entity_view(self, entity_view, **kwargs):
        return self.entity_view_controller.save_entity_view_using_post(entity_view, **kwargs)

    ##################################

    def delete_customer(self, customer_id: CustomerId, **kwargs):
        return self.customer_controller.delete_customer_using_delete(customer_id.id, **kwargs)

    def get_customer_by_id(self, customer_id: CustomerId, **kwargs):
        return self.customer_controller.get_customer_by_id_using_get(customer_id.id, **kwargs)

    def get_customer_title_by_id(self, customer_id: CustomerId, **kwargs):
        return self.customer_controller.get_customer_title_by_id_using_get(customer_id.id, **kwargs)

    def get_customers(self, page_size=100, page=0, **kwargs):
        return self.customer_controller.get_customers_using_get(str(page_size), str(page), **kwargs)

    def get_short_customer_info_by_id(self, customer_id: CustomerId, **kwargs):
        return self.customer_controller.get_short_customer_info_by_id_using_get(customer_id.id, **kwargs)

    def get_tenant_customer(self, customer_title, **kwargs):
        return self.customer_controller.get_tenant_customer_using_get(customer_title, **kwargs)

    def save_customer(self, customer, **kwargs):
        return self.customer_controller.save_customer_using_post(customer, **kwargs)

    ##################################

    def check_updates(self, **kwargs):
        return self.admin_controller.check_updates_using_get(**kwargs)

    def get_admin_settings(self, key, **kwargs):
        return self.admin_controller.get_admin_settings_using_get(key, **kwargs)

    def get_security_settings(self, **kwargs):
        return self.admin_controller.get_security_settings_using_get(**kwargs)

    def save_admin_settings(self, admin_settings, **kwargs):
        return self.admin_controller.save_admin_settings_using_post(admin_settings, **kwargs)

    def save_security_settings(self, security_settings, **kwargs):
        return self.admin_controller.save_security_settings_using_post(security_settings, **kwargs)

    def send_test_mail(self, admin_settings, **kwargs):
        return self.admin_controller.send_test_mail_using_post(admin_settings, **kwargs)

    def send_test_sms(self, message, numberTo, providerConfiguration, **kwargs):
        return self.admin_controller.send_test_sms_using_post(message, numberTo, providerConfiguration, **kwargs)

    ##################################

    def find_by_query(self, query, **kwargs):
        if isinstance(query, DeviceSearchQuery):
            return self.device_controller.find_by_query_using_post1(query, **kwargs)
        elif isinstance(query, EntityViewSearchQuery):
            return self.entity_view_controller.find_by_query_using_post3(query, **kwargs)
        elif isinstance(query, AssetSearchQuery):
            return self.asset_controller.find_by_query_using_post(query, **kwargs)
        elif isinstance(query, EntityRelationsQuery):
            return self.entity_relation_controller.find_by_query_using_post2(query, **kwargs)

    def __load_controllers(self):
        self.admin_controller = AdminControllerApi(self.api_client)
        self.alarm_controller = AlarmControllerApi(self.api_client)
        self.asset_controller = AssetControllerApi(self.api_client)
        self.audit_log_controller = AuditLogControllerApi(self.api_client)
        self.auth_controller = AuthControllerApi(self.api_client)
        self.component_descriptor_controller = ComponentDescriptorControllerApi(self.api_client)
        self.customer_controller = CustomerControllerApi(self.api_client)
        self.dashboard_controller = DashboardControllerApi(self.api_client)
        self.device_api_controller = DeviceApiControllerApi(self.api_client)
        self.device_controller = DeviceControllerApi(self.api_client)
        self.device_profile_controller = DeviceProfileControllerApi(self.api_client)
        self.entity_query_controller = EntityQueryControllerApi(self.api_client)
        self.entity_relation_controller = EntityRelationControllerApi(self.api_client)
        self.entity_view_controller = EntityViewControllerApi(self.api_client)
        self.event_controller = EventControllerApi(self.api_client)
        self.o_auth_2_config_template_controller = OAuth2ConfigTemplateControllerApi(self.api_client)
        self.o_auth_2_controller = OAuth2ControllerApi(self.api_client)
        self.queue_controller = QueueControllerApi(self.api_client)
        self.rpc_controller = RpcControllerApi(self.api_client)
        self.rule_chain_controller = RuleChainControllerApi(self.api_client)
        self.telemetry_controller = TelemetryControllerApi(self.api_client)
        self.tenant_controller = TenantControllerApi(self.api_client)
        self.tenant_profile_controller = TenantProfileControllerApi(self.api_client)
        self.user_controller = UserControllerApi(self.api_client)
        self.widget_type_controller = WidgetTypeControllerApi(self.api_client)
        self.widgets_bundle_controller = WidgetsBundleControllerApi(self.api_client)

    @staticmethod
    def get_type(type):
        return type.entity_type if hasattr(type, "entity_type") else type

    @staticmethod
    def get_id(id):
        return id.id if hasattr(id, "id") else id



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

import pprint
import re  # noqa: F401

import six


class EntityDataSortOrder(object):
    """NOTE: This class is auto generated by the swagger code generator program.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'direction': 'str',
        'key': 'EntityKey'
    }

    attribute_map = {
        'direction': 'direction',
        'key': 'key'
    }

    def __init__(self, direction=None, key=None):  # noqa: E501
        """EntityDataSortOrder - a model defined in Swagger"""  # noqa: E501

        self._direction = None
        self._key = None
        self.discriminator = None

        if direction is not None:
            self.direction = direction
        if key is not None:
            self.key = key

    @property
    def direction(self):
        """Gets the direction of this EntityDataSortOrder.  # noqa: E501


        :return: The direction of this EntityDataSortOrder.  # noqa: E501
        :rtype: str
        """
        return self._direction

    @direction.setter
    def direction(self, direction):
        """Sets the direction of this EntityDataSortOrder.


        :param direction: The direction of this EntityDataSortOrder.  # noqa: E501
        :type: str
        """
        allowed_values = ["ASC", "DESC"]  # noqa: E501
        if direction not in allowed_values:
            raise ValueError(
                "Invalid value for `direction` ({0}), must be one of {1}"  # noqa: E501
                .format(direction, allowed_values)
            )

        self._direction = direction

    @property
    def key(self):
        """Gets the key of this EntityDataSortOrder.  # noqa: E501


        :return: The key of this EntityDataSortOrder.  # noqa: E501
        :rtype: EntityKey
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this EntityDataSortOrder.


        :param key: The key of this EntityDataSortOrder.  # noqa: E501
        :type: EntityKey
        """

        self._key = key

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(EntityDataSortOrder, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, EntityDataSortOrder):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    ThingsBoard REST API

    For instructions how to authorize requests please visit <a href='http://thingsboard.io/docs/reference/rest-api/'>REST API documentation page</a>.  # noqa: E501

    OpenAPI spec version: 2.0
    Contact: info@thingsboard.io
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six


class ReportConfig(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'base_url': 'str',
        'dashboard_id': 'str',
        'name_pattern': 'str',
        'state': 'str',
        'timewindow': 'str',
        'timezone': 'str',
        'type': 'str',
        'use_current_user_credentials': 'bool',
        'use_dashboard_timewindow': 'bool',
        'user_id': 'str'
    }

    attribute_map = {
        'base_url': 'baseUrl',
        'dashboard_id': 'dashboardId',
        'name_pattern': 'namePattern',
        'state': 'state',
        'timewindow': 'timewindow',
        'timezone': 'timezone',
        'type': 'type',
        'use_current_user_credentials': 'useCurrentUserCredentials',
        'use_dashboard_timewindow': 'useDashboardTimewindow',
        'user_id': 'userId'
    }

    def __init__(self, base_url=None, dashboard_id=None, name_pattern=None, state=None, timewindow=None, timezone=None, type=None, use_current_user_credentials=None, use_dashboard_timewindow=None, user_id=None):  # noqa: E501
        """ReportConfig - a model defined in Swagger"""  # noqa: E501

        self._base_url = None
        self._dashboard_id = None
        self._name_pattern = None
        self._state = None
        self._timewindow = None
        self._timezone = None
        self._type = None
        self._use_current_user_credentials = None
        self._use_dashboard_timewindow = None
        self._user_id = None
        self.discriminator = None

        if base_url is not None:
            self.base_url = base_url
        if dashboard_id is not None:
            self.dashboard_id = dashboard_id
        if name_pattern is not None:
            self.name_pattern = name_pattern
        if state is not None:
            self.state = state
        if timewindow is not None:
            self.timewindow = timewindow
        if timezone is not None:
            self.timezone = timezone
        if type is not None:
            self.type = type
        if use_current_user_credentials is not None:
            self.use_current_user_credentials = use_current_user_credentials
        if use_dashboard_timewindow is not None:
            self.use_dashboard_timewindow = use_dashboard_timewindow
        if user_id is not None:
            self.user_id = user_id

    @property
    def base_url(self):
        """Gets the base_url of this ReportConfig.  # noqa: E501


        :return: The base_url of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._base_url

    @base_url.setter
    def base_url(self, base_url):
        """Sets the base_url of this ReportConfig.


        :param base_url: The base_url of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._base_url = base_url

    @property
    def dashboard_id(self):
        """Gets the dashboard_id of this ReportConfig.  # noqa: E501


        :return: The dashboard_id of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._dashboard_id

    @dashboard_id.setter
    def dashboard_id(self, dashboard_id):
        """Sets the dashboard_id of this ReportConfig.


        :param dashboard_id: The dashboard_id of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._dashboard_id = dashboard_id

    @property
    def name_pattern(self):
        """Gets the name_pattern of this ReportConfig.  # noqa: E501


        :return: The name_pattern of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._name_pattern

    @name_pattern.setter
    def name_pattern(self, name_pattern):
        """Sets the name_pattern of this ReportConfig.


        :param name_pattern: The name_pattern of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._name_pattern = name_pattern

    @property
    def state(self):
        """Gets the state of this ReportConfig.  # noqa: E501


        :return: The state of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this ReportConfig.


        :param state: The state of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._state = state

    @property
    def timewindow(self):
        """Gets the timewindow of this ReportConfig.  # noqa: E501


        :return: The timewindow of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._timewindow

    @timewindow.setter
    def timewindow(self, timewindow):
        """Sets the timewindow of this ReportConfig.


        :param timewindow: The timewindow of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._timewindow = timewindow

    @property
    def timezone(self):
        """Gets the timezone of this ReportConfig.  # noqa: E501


        :return: The timezone of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._timezone

    @timezone.setter
    def timezone(self, timezone):
        """Sets the timezone of this ReportConfig.


        :param timezone: The timezone of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._timezone = timezone

    @property
    def type(self):
        """Gets the type of this ReportConfig.  # noqa: E501


        :return: The type of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ReportConfig.


        :param type: The type of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def use_current_user_credentials(self):
        """Gets the use_current_user_credentials of this ReportConfig.  # noqa: E501


        :return: The use_current_user_credentials of this ReportConfig.  # noqa: E501
        :rtype: bool
        """
        return self._use_current_user_credentials

    @use_current_user_credentials.setter
    def use_current_user_credentials(self, use_current_user_credentials):
        """Sets the use_current_user_credentials of this ReportConfig.


        :param use_current_user_credentials: The use_current_user_credentials of this ReportConfig.  # noqa: E501
        :type: bool
        """

        self._use_current_user_credentials = use_current_user_credentials

    @property
    def use_dashboard_timewindow(self):
        """Gets the use_dashboard_timewindow of this ReportConfig.  # noqa: E501


        :return: The use_dashboard_timewindow of this ReportConfig.  # noqa: E501
        :rtype: bool
        """
        return self._use_dashboard_timewindow

    @use_dashboard_timewindow.setter
    def use_dashboard_timewindow(self, use_dashboard_timewindow):
        """Sets the use_dashboard_timewindow of this ReportConfig.


        :param use_dashboard_timewindow: The use_dashboard_timewindow of this ReportConfig.  # noqa: E501
        :type: bool
        """

        self._use_dashboard_timewindow = use_dashboard_timewindow

    @property
    def user_id(self):
        """Gets the user_id of this ReportConfig.  # noqa: E501


        :return: The user_id of this ReportConfig.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this ReportConfig.


        :param user_id: The user_id of this ReportConfig.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ReportConfig, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ReportConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from tb_rest_client32.api_client import ApiClient


class EntityQueryControllerApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def count_entities_by_query_using_post(self, query, **kwargs):  # noqa: E501
        """countEntitiesByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.count_entities_by_query_using_post(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EntityCountQuery query: query (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.count_entities_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
        else:
            (data) = self.count_entities_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
            return data

    def count_entities_by_query_using_post_with_http_info(self, query, **kwargs):  # noqa: E501
        """countEntitiesByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.count_entities_by_query_using_post_with_http_info(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EntityCountQuery query: query (required)
        :return: int
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['query']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):

            params[key] = val
        del params['kwargs']
        # verify the required parameter 'query' is set
        if ('query' not in params or
                params['query'] is None):
            raise ValueError("Missing the required parameter `query` when calling `count_entities_by_query_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'query' in params:
            body_params = params['query']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/entitiesQuery/count', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='int',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def find_alarm_data_by_query_using_post(self, query, **kwargs):  # noqa: E501
        """findAlarmDataByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.find_alarm_data_by_query_using_post(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AlarmDataQuery query: query (required)
        :return: PageDataAlarmData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.find_alarm_data_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
        else:
            (data) = self.find_alarm_data_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
            return data

    def find_alarm_data_by_query_using_post_with_http_info(self, query, **kwargs):  # noqa: E501
        """findAlarmDataByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.find_alarm_data_by_query_using_post_with_http_info(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AlarmDataQuery query: query (required)
        :return: PageDataAlarmData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['query']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):

            params[key] = val
        del params['kwargs']
        # verify the required parameter 'query' is set
        if ('query' not in params or
                params['query'] is None):
            raise ValueError("Missing the required parameter `query` when calling `find_alarm_data_by_query_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'query' in params:
            body_params = params['query']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/alarmsQuery/find', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PageDataAlarmData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def find_entity_data_by_query_using_post(self, query, **kwargs):  # noqa: E501
        """findEntityDataByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.find_entity_data_by_query_using_post(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EntityDataQuery query: query (required)
        :return: PageDataEntityData
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.find_entity_data_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
        else:
            (data) = self.find_entity_data_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
            return data

    def find_entity_data_by_query_using_post_with_http_info(self, query, **kwargs):  # noqa: E501
        """findEntityDataByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.find_entity_data_by_query_using_post_with_http_info(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param EntityDataQuery query: query (required)
        :return: PageDataEntityData
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['query']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):

            params[key] = val
        del params['kwargs']
        # verify the required parameter 'query' is set
        if ('query' not in params or
                params['query'] is None):
            raise ValueError("Missing the required parameter `query` when calling `find_entity_data_by_query_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'query' in params:
            body_params = params['query']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/entitiesQuery/find', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PageDataEntityData',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#

import pprint
import re  # noqa: F401

import six


class UserPasswordPolicy(object):
    """NOTE: This class is auto generated by the swagger code generator program.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'minimum_digits': 'int',
        'minimum_length': 'int',
        'minimum_lowercase_letters': 'int',
        'minimum_special_characters': 'int',
        'minimum_uppercase_letters': 'int',
        'password_expiration_period_days': 'int',
        'password_reuse_frequency_days': 'int'
    }

    attribute_map = {
        'minimum_digits': 'minimumDigits',
        'minimum_length': 'minimumLength',
        'minimum_lowercase_letters': 'minimumLowercaseLetters',
        'minimum_special_characters': 'minimumSpecialCharacters',
        'minimum_uppercase_letters': 'minimumUppercaseLetters',
        'password_expiration_period_days': 'passwordExpirationPeriodDays',
        'password_reuse_frequency_days': 'passwordReuseFrequencyDays'
    }

    def __init__(self, minimum_digits=None, minimum_length=None, minimum_lowercase_letters=None, minimum_special_characters=None, minimum_uppercase_letters=None, password_expiration_period_days=None, password_reuse_frequency_days=None):  # noqa: E501
        """UserPasswordPolicy - a model defined in Swagger"""  # noqa: E501

        self._minimum_digits = None
        self._minimum_length = None
        self._minimum_lowercase_letters = None
        self._minimum_special_characters = None
        self._minimum_uppercase_letters = None
        self._password_expiration_period_days = None
        self._password_reuse_frequency_days = None
        self.discriminator = None

        if minimum_digits is not None:
            self.minimum_digits = minimum_digits
        if minimum_length is not None:
            self.minimum_length = minimum_length
        if minimum_lowercase_letters is not None:
            self.minimum_lowercase_letters = minimum_lowercase_letters
        if minimum_special_characters is not None:
            self.minimum_special_characters = minimum_special_characters
        if minimum_uppercase_letters is not None:
            self.minimum_uppercase_letters = minimum_uppercase_letters
        if password_expiration_period_days is not None:
            self.password_expiration_period_days = password_expiration_period_days
        if password_reuse_frequency_days is not None:
            self.password_reuse_frequency_days = password_reuse_frequency_days

    @property
    def minimum_digits(self):
        """Gets the minimum_digits of this UserPasswordPolicy.  # noqa: E501


        :return: The minimum_digits of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._minimum_digits

    @minimum_digits.setter
    def minimum_digits(self, minimum_digits):
        """Sets the minimum_digits of this UserPasswordPolicy.


        :param minimum_digits: The minimum_digits of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._minimum_digits = minimum_digits

    @property
    def minimum_length(self):
        """Gets the minimum_length of this UserPasswordPolicy.  # noqa: E501


        :return: The minimum_length of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._minimum_length

    @minimum_length.setter
    def minimum_length(self, minimum_length):
        """Sets the minimum_length of this UserPasswordPolicy.


        :param minimum_length: The minimum_length of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._minimum_length = minimum_length

    @property
    def minimum_lowercase_letters(self):
        """Gets the minimum_lowercase_letters of this UserPasswordPolicy.  # noqa: E501


        :return: The minimum_lowercase_letters of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._minimum_lowercase_letters

    @minimum_lowercase_letters.setter
    def minimum_lowercase_letters(self, minimum_lowercase_letters):
        """Sets the minimum_lowercase_letters of this UserPasswordPolicy.


        :param minimum_lowercase_letters: The minimum_lowercase_letters of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._minimum_lowercase_letters = minimum_lowercase_letters

    @property
    def minimum_special_characters(self):
        """Gets the minimum_special_characters of this UserPasswordPolicy.  # noqa: E501


        :return: The minimum_special_characters of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._minimum_special_characters

    @minimum_special_characters.setter
    def minimum_special_characters(self, minimum_special_characters):
        """Sets the minimum_special_characters of this UserPasswordPolicy.


        :param minimum_special_characters: The minimum_special_characters of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._minimum_special_characters = minimum_special_characters

    @property
    def minimum_uppercase_letters(self):
        """Gets the minimum_uppercase_letters of this UserPasswordPolicy.  # noqa: E501


        :return: The minimum_uppercase_letters of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._minimum_uppercase_letters

    @minimum_uppercase_letters.setter
    def minimum_uppercase_letters(self, minimum_uppercase_letters):
        """Sets the minimum_uppercase_letters of this UserPasswordPolicy.


        :param minimum_uppercase_letters: The minimum_uppercase_letters of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._minimum_uppercase_letters = minimum_uppercase_letters

    @property
    def password_expiration_period_days(self):
        """Gets the password_expiration_period_days of this UserPasswordPolicy.  # noqa: E501


        :return: The password_expiration_period_days of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._password_expiration_period_days

    @password_expiration_period_days.setter
    def password_expiration_period_days(self, password_expiration_period_days):
        """Sets the password_expiration_period_days of this UserPasswordPolicy.


        :param password_expiration_period_days: The password_expiration_period_days of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._password_expiration_period_days = password_expiration_period_days

    @property
    def password_reuse_frequency_days(self):
        """Gets the password_reuse_frequency_days of this UserPasswordPolicy.  # noqa: E501


        :return: The password_reuse_frequency_days of this UserPasswordPolicy.  # noqa: E501
        :rtype: int
        """
        return self._password_reuse_frequency_days

    @password_reuse_frequency_days.setter
    def password_reuse_frequency_days(self, password_reuse_frequency_days):
        """Sets the password_reuse_frequency_days of this UserPasswordPolicy.


        :param password_reuse_frequency_days: The password_reuse_frequency_days of this UserPasswordPolicy.  # noqa: E501
        :type: int
        """

        self._password_reuse_frequency_days = password_reuse_frequency_days

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(UserPasswordPolicy, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UserPasswordPolicy):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#
#

import logging
# Importing models and REST client class from Community Edition version
from tb_rest_client32.rest_client_ce import *
from tb_rest_client32.rest import ApiException


logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s - %(levelname)s - %(module)s - %(lineno)d - %(message)s',
                    datefmt='%Y-%m-%d %H:%M:%S')

# ThingsBoard REST API URL
url = "http://localhost:8080"

# Default Tenant Administrator credentials
username = "tenant@thingsboard.org"
password = "tenant"


# Creating the REST client object with context manager to get auto token refresh
with RestClientCE(base_url=url) as rest_client:
    try:
        # Auth with credentials
        rest_client.login(username=username, password=password)

        # Creating an Asset
        asset = Asset(name="Building 1", type="building")
        asset = rest_client.save_asset(asset)

        logging.info("Asset was created:\n%r\n", asset)

        # creating a Device
        device = Device(name="Thermometer 11", type="thermometer")
        device = rest_client.save_device(device)

        logging.info(" Device was created:\n%r\n", device)

        # Creating relations from device to asset
        relation = EntityRelation(_from=asset.id, to=device.id, type="Contains")
        relation = rest_client.save_relation(relation)

        logging.info(" Relation was created:\n%r\n", relation)
    except ApiException as e:
        logging.exception(e)




# coding: utf-8
#      Copyright 2020. ThingsBoard
#  #
#      Licensed under the Apache License, Version 2.0 (the "License");
#      you may not use this file except in compliance with the License.
#      You may obtain a copy of the License at
#  #
#          http://www.apache.org/licenses/LICENSE-2.0
#  #
#      Unless required by applicable law or agreed to in writing, software
#      distributed under the License is distributed on an "AS IS" BASIS,
#      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#      See the License for the specific language governing permissions and
#      limitations under the License.
#
#

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from tb_rest_client32.api_client import ApiClient


class AssetControllerApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def assign_asset_to_customer_using_post(self, customer_id, asset_id, **kwargs):  # noqa: E501
        """assignAssetToCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.assign_asset_to_customer_using_post(customer_id, asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.assign_asset_to_customer_using_post_with_http_info(customer_id, asset_id, **kwargs)  # noqa: E501
        else:
            (data) = self.assign_asset_to_customer_using_post_with_http_info(customer_id, asset_id, **kwargs)  # noqa: E501
            return data

    def assign_asset_to_customer_using_post_with_http_info(self, customer_id, asset_id, **kwargs):  # noqa: E501
        """assignAssetToCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.assign_asset_to_customer_using_post_with_http_info(customer_id, asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['customer_id', 'asset_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'customer_id' is set
        if ('customer_id' not in params or
                params['customer_id'] is None):
            raise ValueError("Missing the required parameter `customer_id` when calling `assign_asset_to_customer_using_post`")  # noqa: E501
        # verify the required parameter 'asset_id' is set
        if ('asset_id' not in params or
                params['asset_id'] is None):
            raise ValueError("Missing the required parameter `asset_id` when calling `assign_asset_to_customer_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'customer_id' in params:
            path_params['customerId'] = params['customer_id']  # noqa: E501
        if 'asset_id' in params:
            path_params['assetId'] = params['asset_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/{customerId}/asset/{assetId}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Asset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def assign_asset_to_public_customer_using_post(self, asset_id, **kwargs):  # noqa: E501
        """assignAssetToPublicCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.assign_asset_to_public_customer_using_post(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.assign_asset_to_public_customer_using_post_with_http_info(asset_id, **kwargs)  # noqa: E501
        else:
            (data) = self.assign_asset_to_public_customer_using_post_with_http_info(asset_id, **kwargs)  # noqa: E501
            return data

    def assign_asset_to_public_customer_using_post_with_http_info(self, asset_id, **kwargs):  # noqa: E501
        """assignAssetToPublicCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.assign_asset_to_public_customer_using_post_with_http_info(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_id' is set
        if ('asset_id' not in params or
                params['asset_id'] is None):
            raise ValueError("Missing the required parameter `asset_id` when calling `assign_asset_to_public_customer_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'asset_id' in params:
            path_params['assetId'] = params['asset_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/public/asset/{assetId}', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Asset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def delete_asset_using_delete(self, asset_id, **kwargs):  # noqa: E501
        """deleteAsset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.delete_asset_using_delete(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.delete_asset_using_delete_with_http_info(asset_id, **kwargs)  # noqa: E501
        else:
            (data) = self.delete_asset_using_delete_with_http_info(asset_id, **kwargs)  # noqa: E501
            return data

    def delete_asset_using_delete_with_http_info(self, asset_id, **kwargs):  # noqa: E501
        """deleteAsset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.delete_asset_using_delete_with_http_info(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_id' is set
        if ('asset_id' not in params or
                params['asset_id'] is None):
            raise ValueError("Missing the required parameter `asset_id` when calling `delete_asset_using_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'asset_id' in params:
            path_params['assetId'] = params['asset_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/asset/{assetId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def find_by_query_using_post(self, query, **kwargs):  # noqa: E501
        """findByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.find_by_query_using_post(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AssetSearchQuery query: query (required)
        :return: list[Asset]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.find_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
        else:
            (data) = self.find_by_query_using_post_with_http_info(query, **kwargs)  # noqa: E501
            return data

    def find_by_query_using_post_with_http_info(self, query, **kwargs):  # noqa: E501
        """findByQuery  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.find_by_query_using_post_with_http_info(query, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param AssetSearchQuery query: query (required)
        :return: list[Asset]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['query']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'query' is set
        if ('query' not in params or
                params['query'] is None):
            raise ValueError("Missing the required parameter `query` when calling `find_by_query_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'query' in params:
            body_params = params['query']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/assets', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[Asset]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_asset_by_id_using_get(self, asset_id, **kwargs):  # noqa: E501
        """getAssetById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_asset_by_id_using_get(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_asset_by_id_using_get_with_http_info(asset_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_asset_by_id_using_get_with_http_info(asset_id, **kwargs)  # noqa: E501
            return data

    def get_asset_by_id_using_get_with_http_info(self, asset_id, **kwargs):  # noqa: E501
        """getAssetById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_asset_by_id_using_get_with_http_info(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_id' is set
        if ('asset_id' not in params or
                params['asset_id'] is None):
            raise ValueError("Missing the required parameter `asset_id` when calling `get_asset_by_id_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'asset_id' in params:
            path_params['assetId'] = params['asset_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/asset/{assetId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Asset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_asset_info_by_id_using_get(self, asset_id, **kwargs):  # noqa: E501
        """getAssetInfoById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_asset_info_by_id_using_get(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: AssetInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_asset_info_by_id_using_get_with_http_info(asset_id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_asset_info_by_id_using_get_with_http_info(asset_id, **kwargs)  # noqa: E501
            return data

    def get_asset_info_by_id_using_get_with_http_info(self, asset_id, **kwargs):  # noqa: E501
        """getAssetInfoById  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_asset_info_by_id_using_get_with_http_info(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: AssetInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_id' is set
        if ('asset_id' not in params or
                params['asset_id'] is None):
            raise ValueError("Missing the required parameter `asset_id` when calling `get_asset_info_by_id_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'asset_id' in params:
            path_params['assetId'] = params['asset_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/asset/info/{assetId}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='AssetInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_asset_types_using_get(self, **kwargs):  # noqa: E501
        """getAssetTypes  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_asset_types_using_get(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[EntitySubtype]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_asset_types_using_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_asset_types_using_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_asset_types_using_get_with_http_info(self, **kwargs):  # noqa: E501
        """getAssetTypes  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_asset_types_using_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[EntitySubtype]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/asset/types', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[EntitySubtype]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_assets_by_ids_using_get(self, asset_ids, **kwargs):  # noqa: E501
        """getAssetsByIds  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_assets_by_ids_using_get(asset_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_ids: assetIds (required)
        :return: list[Asset]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_assets_by_ids_using_get_with_http_info(asset_ids, **kwargs)  # noqa: E501
        else:
            (data) = self.get_assets_by_ids_using_get_with_http_info(asset_ids, **kwargs)  # noqa: E501
            return data

    def get_assets_by_ids_using_get_with_http_info(self, asset_ids, **kwargs):  # noqa: E501
        """getAssetsByIds  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_assets_by_ids_using_get_with_http_info(asset_ids, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_ids: assetIds (required)
        :return: list[Asset]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_ids']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_ids' is set
        if ('asset_ids' not in params or
                params['asset_ids'] is None):
            raise ValueError("Missing the required parameter `asset_ids` when calling `get_assets_by_ids_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'asset_ids' in params:
            query_params.append(('assetIds', params['asset_ids']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/assets{?assetIds}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='list[Asset]',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_customer_asset_infos_using_get(self, customer_id, page_size, page, **kwargs):  # noqa: E501
        """getCustomerAssetInfos  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_customer_asset_infos_using_get(customer_id, page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAssetInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_customer_asset_infos_using_get_with_http_info(customer_id, page_size, page, **kwargs)  # noqa: E501
        else:
            (data) = self.get_customer_asset_infos_using_get_with_http_info(customer_id, page_size, page, **kwargs)  # noqa: E501
            return data

    def get_customer_asset_infos_using_get_with_http_info(self, customer_id, page_size, page, **kwargs):  # noqa: E501
        """getCustomerAssetInfos  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_customer_asset_infos_using_get_with_http_info(customer_id, page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAssetInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['customer_id', 'page_size', 'page', 'type', 'text_search', 'sort_property', 'sort_order']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'customer_id' is set
        if ('customer_id' not in params or
                params['customer_id'] is None):
            raise ValueError("Missing the required parameter `customer_id` when calling `get_customer_asset_infos_using_get`")  # noqa: E501
        # verify the required parameter 'page_size' is set
        if ('page_size' not in params or
                params['page_size'] is None):
            raise ValueError("Missing the required parameter `page_size` when calling `get_customer_asset_infos_using_get`")  # noqa: E501
        # verify the required parameter 'page' is set
        if ('page' not in params or
                params['page'] is None):
            raise ValueError("Missing the required parameter `page` when calling `get_customer_asset_infos_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'customer_id' in params:
            path_params['customerId'] = params['customer_id']  # noqa: E501

        query_params = []
        if 'type' in params:
            query_params.append(('type', params['type']))  # noqa: E501
        if 'text_search' in params:
            query_params.append(('textSearch', params['text_search']))  # noqa: E501
        if 'sort_property' in params:
            query_params.append(('sortProperty', params['sort_property']))  # noqa: E501
        if 'sort_order' in params:
            query_params.append(('sortOrder', params['sort_order']))  # noqa: E501
        if 'page_size' in params:
            query_params.append(('pageSize', params['page_size']))  # noqa: E501
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/{customerId}/assetInfos{?type,textSearch,sortProperty,sortOrder,pageSize,page}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PageDataAssetInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_customer_assets_using_get(self, customer_id, page_size, page, **kwargs):  # noqa: E501
        """getCustomerAssets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_customer_assets_using_get(customer_id, page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAsset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_customer_assets_using_get_with_http_info(customer_id, page_size, page, **kwargs)  # noqa: E501
        else:
            (data) = self.get_customer_assets_using_get_with_http_info(customer_id, page_size, page, **kwargs)  # noqa: E501
            return data

    def get_customer_assets_using_get_with_http_info(self, customer_id, page_size, page, **kwargs):  # noqa: E501
        """getCustomerAssets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_customer_assets_using_get_with_http_info(customer_id, page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str customer_id: customerId (required)
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAsset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['customer_id', 'page_size', 'page', 'type', 'text_search', 'sort_property', 'sort_order']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'customer_id' is set
        if ('customer_id' not in params or
                params['customer_id'] is None):
            raise ValueError("Missing the required parameter `customer_id` when calling `get_customer_assets_using_get`")  # noqa: E501
        # verify the required parameter 'page_size' is set
        if ('page_size' not in params or
                params['page_size'] is None):
            raise ValueError("Missing the required parameter `page_size` when calling `get_customer_assets_using_get`")  # noqa: E501
        # verify the required parameter 'page' is set
        if ('page' not in params or
                params['page'] is None):
            raise ValueError("Missing the required parameter `page` when calling `get_customer_assets_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'customer_id' in params:
            path_params['customerId'] = params['customer_id']  # noqa: E501

        query_params = []
        if 'type' in params:
            query_params.append(('type', params['type']))  # noqa: E501
        if 'text_search' in params:
            query_params.append(('textSearch', params['text_search']))  # noqa: E501
        if 'sort_property' in params:
            query_params.append(('sortProperty', params['sort_property']))  # noqa: E501
        if 'sort_order' in params:
            query_params.append(('sortOrder', params['sort_order']))  # noqa: E501
        if 'page_size' in params:
            query_params.append(('pageSize', params['page_size']))  # noqa: E501
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/{customerId}/assets{?type,textSearch,sortProperty,sortOrder,pageSize,page}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PageDataAsset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_tenant_asset_infos_using_get(self, page_size, page, **kwargs):  # noqa: E501
        """getTenantAssetInfos  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_tenant_asset_infos_using_get(page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAssetInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_tenant_asset_infos_using_get_with_http_info(page_size, page, **kwargs)  # noqa: E501
        else:
            (data) = self.get_tenant_asset_infos_using_get_with_http_info(page_size, page, **kwargs)  # noqa: E501
            return data

    def get_tenant_asset_infos_using_get_with_http_info(self, page_size, page, **kwargs):  # noqa: E501
        """getTenantAssetInfos  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_ce.get_tenant_asset_infos_using_get_with_http_info(page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAssetInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['page_size', 'page', 'type', 'text_search', 'sort_property', 'sort_order']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'page_size' is set
        if ('page_size' not in params or
                params['page_size'] is None):
            raise ValueError("Missing the required parameter `page_size` when calling `get_tenant_asset_infos_using_get`")  # noqa: E501
        # verify the required parameter 'page' is set
        if ('page' not in params or
                params['page'] is None):
            raise ValueError("Missing the required parameter `page` when calling `get_tenant_asset_infos_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'type' in params:
            query_params.append(('type', params['type']))  # noqa: E501
        if 'text_search' in params:
            query_params.append(('textSearch', params['text_search']))  # noqa: E501
        if 'sort_property' in params:
            query_params.append(('sortProperty', params['sort_property']))  # noqa: E501
        if 'sort_order' in params:
            query_params.append(('sortOrder', params['sort_order']))  # noqa: E501
        if 'page_size' in params:
            query_params.append(('pageSize', params['page_size']))  # noqa: E501
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/tenant/assetInfos{?type,textSearch,sortProperty,sortOrder,pageSize,page}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PageDataAssetInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_tenant_asset_using_get(self, asset_name, **kwargs):  # noqa: E501
        """getTenantAsset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_asset_using_get(asset_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_name: assetName (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_tenant_asset_using_get_with_http_info(asset_name, **kwargs)  # noqa: E501
        else:
            (data) = self.get_tenant_asset_using_get_with_http_info(asset_name, **kwargs)  # noqa: E501
            return data

    def get_tenant_asset_using_get_with_http_info(self, asset_name, **kwargs):  # noqa: E501
        """getTenantAsset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_asset_using_get_with_http_info(asset_name, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_name: assetName (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_name']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_name' is set
        if ('asset_name' not in params or
                params['asset_name'] is None):
            raise ValueError("Missing the required parameter `asset_name` when calling `get_tenant_asset_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'asset_name' in params:
            query_params.append(('assetName', params['asset_name']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/tenant/assets{?assetName}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Asset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_tenant_assets_using_get(self, page_size, page, **kwargs):  # noqa: E501
        """getTenantAssets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_assets_using_get(page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAsset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_tenant_assets_using_get_with_http_info(page_size, page, **kwargs)  # noqa: E501
        else:
            (data) = self.get_tenant_assets_using_get_with_http_info(page_size, page, **kwargs)  # noqa: E501
            return data

    def get_tenant_assets_using_get_with_http_info(self, page_size, page, **kwargs):  # noqa: E501
        """getTenantAssets  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.get_tenant_assets_using_get_with_http_info(page_size, page, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str page_size: pageSize (required)
        :param str page: page (required)
        :param str type: type
        :param str text_search: textSearch
        :param str sort_property: sortProperty
        :param str sort_order: sortOrder
        :return: PageDataAsset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['page_size', 'page', 'type', 'text_search', 'sort_property', 'sort_order']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'page_size' is set
        if ('page_size' not in params or
                params['page_size'] is None):
            raise ValueError("Missing the required parameter `page_size` when calling `get_tenant_assets_using_get`")  # noqa: E501
        # verify the required parameter 'page' is set
        if ('page' not in params or
                params['page'] is None):
            raise ValueError("Missing the required parameter `page` when calling `get_tenant_assets_using_get`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'type' in params:
            query_params.append(('type', params['type']))  # noqa: E501
        if 'text_search' in params:
            query_params.append(('textSearch', params['text_search']))  # noqa: E501
        if 'sort_property' in params:
            query_params.append(('sortProperty', params['sort_property']))  # noqa: E501
        if 'sort_order' in params:
            query_params.append(('sortOrder', params['sort_order']))  # noqa: E501
        if 'page_size' in params:
            query_params.append(('pageSize', params['page_size']))  # noqa: E501
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/tenant/assets{?type,textSearch,sortProperty,sortOrder,pageSize,page}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='PageDataAsset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def save_asset_using_post(self, asset, **kwargs):  # noqa: E501
        """saveAsset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.save_asset_using_post(asset, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Asset asset: asset (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.save_asset_using_post_with_http_info(asset, **kwargs)  # noqa: E501
        else:
            (data) = self.save_asset_using_post_with_http_info(asset, **kwargs)  # noqa: E501
            return data

    def save_asset_using_post_with_http_info(self, asset, **kwargs):  # noqa: E501
        """saveAsset  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.save_asset_using_post_with_http_info(asset, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param Asset asset: asset (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset' is set
        if ('asset' not in params or
                params['asset'] is None):
            raise ValueError("Missing the required parameter `asset` when calling `save_asset_using_post`")  # noqa: E501

        collection_formats = {}

        path_params = {}

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'asset' in params:
            body_params = params['asset']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/asset', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Asset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def unassign_asset_from_customer_using_delete(self, asset_id, **kwargs):  # noqa: E501
        """unassignAssetFromCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.unassign_asset_from_customer_using_delete(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.unassign_asset_from_customer_using_delete_with_http_info(asset_id, **kwargs)  # noqa: E501
        else:
            (data) = self.unassign_asset_from_customer_using_delete_with_http_info(asset_id, **kwargs)  # noqa: E501
            return data

    def unassign_asset_from_customer_using_delete_with_http_info(self, asset_id, **kwargs):  # noqa: E501
        """unassignAssetFromCustomer  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api_pe.unassign_asset_from_customer_using_delete_with_http_info(asset_id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str asset_id: assetId (required)
        :return: Asset
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['asset_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'asset_id' is set
        if ('asset_id' not in params or
                params['asset_id'] is None):
            raise ValueError("Missing the required parameter `asset_id` when calling `unassign_asset_from_customer_using_delete`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'asset_id' in params:
            path_params['assetId'] = params['asset_id']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['*/*'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['X-Authorization']  # noqa: E501

        return self.api_client.call_api(
            '/api/customer/asset/{assetId}', 'DELETE',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='Asset',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)
